
package bluetoothtest;

import java.lang.*;
import java.io.*;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;
import javax.bluetooth.*;
import javax.microedition.lcdui.game.GameCanvas;
import javax.microedition.lcdui.game.Sprite;

public class PlayerClient extends BlueTMIDlet {
    private OutputStream out;
    private boolean playerMoved;
    private int cooldown = 0;
    private static final int COOLDOWN_TIME = 0;
    private static final int quantum = 5;
    private static final int frameRate = 25;
    private static final int msPerFrame = 1000 / frameRate;
    
    private void handleInput() {
        int dX = 0;
        int dY = 0;
        int keyStates = gCanvas.getKeyStates();
        if ((keyStates & GameCanvas.LEFT_PRESSED) !=0 )
            dX--;
        if ((keyStates & GameCanvas.RIGHT_PRESSED) !=0 )
            dX++;
        if ((keyStates & GameCanvas.UP_PRESSED) !=0 )
            dY--;
        if ((keyStates & GameCanvas.DOWN_PRESSED) !=0 )
            dY++;
        playerMoved = (dX!=0 || dY!=0);
        
        getPlayer().move(quantum*dX, quantum*dY);        
    }
    
    
    private void sendMessages() throws IOException {
        if (cooldown > 0) {
            cooldown--;
        } else {
            if (playerMoved) {
                out.write(getPlayer().getX());
                //out.write(",".getBytes());
                out.write(getPlayer().getY());
                //out.write(":".getBytes());
                out.flush();
                cooldown = COOLDOWN_TIME;
            }
        }
    }
    
    private void redrawCanvas() {
        gCanvas.redraw();
    }
    
    
    /**
     * Connects to the server and sends 'Hello, World'
     * to the server.
     */
    public void run() {
        getPlayer().setPosition(gCanvas.getWidth()/2, gCanvas.getHeight()/2);
        playerMoved = true;
        
        try {
            // Retrieve the connection string to connect to the server
            LocalDevice local = LocalDevice.getLocalDevice();
            DiscoveryAgent agent = local.getDiscoveryAgent();

            String connString = agent.selectService(
                new UUID("86b4d249fb8844d6a756ec265dd1f6a3", false),
                ServiceRecord.NOAUTHENTICATE_NOENCRYPT, false);

            if (connString != null) {
                try {
                    // Connect to the server and send 'Hello, World'
                    StreamConnection conn = (StreamConnection)
                        Connector.open(connString);
                    out = conn.openOutputStream();
                    
                    //Game Loop
                    sendMessages();
                    long startTime = 0;//System.currentTimeMillis();
                    for(;;) {
                        startTime = System.currentTimeMillis();
                        //System.out.println("Start: " + startTime);
                        handleInput();
                        sendMessages();
                        redrawCanvas();
                        
                        if (getPlayer().getX() > gCanvas.getWidth())
                            break;
                        
                        long timeRem = msPerFrame - (System.currentTimeMillis() - startTime);
                        try {
                            Thread.sleep(timeRem);
                        } catch (InterruptedException ex) {}
                    }
                    
                    System.out.println("Done!");
                    
                    out.close();
                    conn.close();

                    System.out.println("Message sent correctly");
                } catch (IOException e) {
                    System.out.println("IO Exception: " + e.getMessage());
                }
            } else {
                System.out.println("Unable to locate service");
            }
        } catch (BluetoothStateException e) {
            System.out.println("BluetoothStateException: " + e.getMessage());
        }
    }
}
